-- DESCRIBING SECTION =============================================================================
-------------------------------------------------------------------------------- 
-- 1. SOFTWAROVA s.r.o.
-- (c) 2012
--------------------------------------------------------------------------------
-- TRANSACTIONAL PARALLEL PROCESSING
--------------------------------------------------------------------------------
-- Requirements:
-- Oracle Database Enterprise version >= 11g
-- CPU >= 1, Mem, Cluster
-- Sufficient size of REDO LOG is required.
--------------------------------------------------------------------------------
-- Run as user defined in TPP_OWNER
-- =============================================================================
------------- S C R I P T    N A M E -----------
DEF patch_type   = du
DEF patch_number = 00000001
DEF step_order   = 03
DEF rest_of_name = _install_tpp
------------- S C R I P T    I N F O -----------
DEF description     = 'Installation script TPP - DDL'
DEF version         = 1.0
DEF created_by      = '1SW'
DEF create_date     = 07.06.2012
DEF affected_apps   = 'TPP'
REM for             = SQL*PLUS
REM parameters      = none

----------- S C R I P T    H I S T O R Y --------
REM     Version   When             Who               What
REM     1.0       2012/05/25       Petr Jezek        Creation
-- INITIAL UNIVERSAL SECTION ====================================================

@../du00000001_setting_env_tpp.sql

DEF script_name  = &patch_type&patch_number._&step_order&rest_of_name
Set SERVEROUTPUT On Size 1000000
SPOOL &script_name..log
Set VERIFY Off
Set TRIMSPOOL On
PROMPT
PROMPT *************************************************************************
PROMPT SCRIPT FILE: &script_name..sql
PROMPT LOG    FILE: &script_name..log
PROMPT SCRIPT DESC: &description
PROMPT SCRIPT VER.: &version
PROMPT *************************************************************************
PROMPT
PROMPT Affected Application(s) : &affected_apps
PROMPT Executing ...
PROMPT
-- EXECUTIVE SECTION ==============================================================================
Set FEEDBACK Off
Declare
  v_db_user VARCHAR2(500);
  v_net_user VARCHAR2(500);
Begin 
   -- Common actions
   SELECT 'Database user: ' || user || '@' || sys_context('USERENV', 'DB_NAME')
     INTO v_db_user 
     FROM dual;
   SELECT 'Network user: ' || sys_context('USERENV', 'OS_USER') || '@' || sys_context('USERENV', 'HOST')
     INTO v_net_user 
     FROM dual;
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('Start ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.put_line(v_db_user);
   DBMS_OUTPUT.put_line(v_net_user);   
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('	') ;
End; 
/
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

prompt create table trace_log
create table trace_log (
  id             number,
  dt             timestamp,
  severity       number,
  procedure_name varchar2(2000),
  description    varchar2(4000),
  primary key (id)
)
tablespace &&MEDIUM_DATA;

comment on table trace_log is
'Log of processing for debug'
;
comment on column trace_log.id             is 'Identification to keep sequence of records in log';
comment on column trace_log.dt             is 'Datetime of logged event';
comment on column trace_log.severity       is 'Severity level';
comment on column trace_log.procedure_name is 'Name of procedure';
comment on column trace_log.description    is 'Value of the debug message';

create sequence seq_trace_log;

grant select on trace_log to &&TPP_USER;


prompt create table transaction_operation
create table transaction_operation (
  task_name varchar2(128 byte) not null,
  primary key (task_name)
)
tablespace &&SMALL_DATA;

comment on table transaction_operation is
'Into this table is writen the test record to idenify commit/rollback operation'
;
comment on column transaction_operation.task_name is 'Name of the task';

prompt create table task
create table task(
  task_name          varchar2(128 byte) not null,
  parallel_level     number,
  to_execute         varchar2(4000),
  job_name           varchar2(4000),
  operation          varchar2(100),
  primary key (task_name)
)
tablespace &&SMALL_DATA;

comment on table task is
'evidence of the tasks'
;
comment on column task.task_name is 'Name of the task';
comment on column task.parallel_level is 'Number of required processes';
comment on column task.task_name is 'The statement to execute for this task';
comment on column task.job_name is 'Name of the job to run for the task';
comment on column task.operation is 'Result of operation commit or rollback';

----
prompt create table task_chunk
create table task_chunk(
  task_name          varchar2(128 byte) not null,
  chunk_id           number not null,
  status             varchar2(100 byte),
  start_rowid        rowid,
  end_rowid          rowid,
  start_id           number,
  end_id             number,
  error_text         varchar2(4000 char)
)
tablespace &&SMALL_DATA;

comment on table task_chunk is
'evidence of the tasks'
;
comment on column task_chunk.task_name    is 'Name of the task';
comment on column task_chunk.chunk_id     is 'Chunk identification';
comment on column task_chunk.status       is 'Status of the chunk';
comment on column task_chunk.start_rowid  is 'Start rowid';
comment on column task_chunk.end_rowid    is 'End rowid';
comment on column task_chunk.start_id     is 'Start id';
comment on column task_chunk.end_id       is 'End id';
comment on column task_chunk.error_text   is 'Text of error when it raise';

create table conf_param
(
  name         varchar2(1000 char)  not null,
  val          varchar2(4000 char),
  description  varchar2(2000 char)
)
tablespace &&SMALL_DATA;

comment on  table conf_param      is 'Configuration of parameters';
comment on column conf_param.name is 'Primary key - name of configurable variable.';
comment on column conf_param.val  is 'Value of configurable variable.';




prompt create table patches
create table patches
(
   seq_number    number(6)
  ,patch_type    varchar2(2 byte)
  ,patch_number  number(5)
  ,script_name   varchar2(200 byte)
  ,created_by    varchar2(50 byte)
  ,create_date   date
  ,applied_by    varchar2(50 byte)
  ,apply_date    date
  ,description   varchar2(2000 byte)
)
tablespace &&SMALL_DATA;

comment on table patches is
'Evidentary of release'
;
comment on column patches.seq_number    is 'Sequence number of release';
comment on column patches.patch_type    is 'Type of patch';
comment on column patches.patch_number  is 'Number of the patch';
comment on column patches.script_name   is 'Script name';
comment on column patches.created_by    is 'Author of the patch';
comment on column patches.create_date   is 'Date of ceration of the patch';
comment on column patches.applied_by    is 'Applied by Oracle user';
comment on column patches.apply_date    is 'Date of apply';
comment on column patches.description   is 'Description of the patch';


prompt create sequence seq_patches
create sequence seq_patches
  start with 1000
  maxvalue 9999999999999999999999999999
  minvalue 1
  nocycle
  cache 20
  noorder;

set def off
prompt create of package tpp_log
@@tpp_log.pks
@@tpp_log.pkb

prompt create of package tpp_internal
@@tpp_internal.pks
@@tpp_internal.pkb


prompt create of package tpp
@@tpp.pks
@@tpp.pkb



set def on
grant execute on tpp to &&TPP_USER;


declare
  x number;
begin
  sys.dbms_job.submit
    (
      job        => x
     ,what       => 'begin tpp_log.clean_old_logs; end;'
     ,next_date  => trunc(sysdate)
     ,interval   => 'trunc(sysdate+7)'
     ,no_parse   => false
    );
--:jobnumber := to_char(x);
end;
/


-- create job to clean record older then 90 days
-- delete from trace_log where


insert into conf_param (name, val, description) values ('LOG_LEVEL', '2', 'Log level for trace_log table. 0 - No log, 1 - Log only fatal error, 2 - Log error and fatal error, 3 - Log warning, errors and fatal error, 4 - Log info, warning, errors and fatal error, 5 - Log debug, info, warning, errors and fatal error');
insert into conf_param (name, val, description) values ('CLEAN_OLDER_THEN_X_DAYS', '90', 'The value is defined interval for maintaince job to clean old data in trace_log table');


Set DEF On

PROMPT *************************************************************************
BEGIN

-- FINAL UNIVERSAL SECTION ========================================================================

   DBMS_OUTPUT.PUT_LINE ('Log - Insert into patches');
    INSERT INTO patches
          (
           seq_number,
           patch_type,
           patch_number,
           script_name,
           created_by,
           create_date,
           applied_by,
           apply_date,
           description
          )
    VALUES
          (
           seq_patches.NEXTVAL,
           DECODE('&patch_type','PR','AN','&patch_type'),
           TO_NUMBER('&patch_number'),
           '&script_name',
           '&created_by',
           TO_DATE('&create_date','DD.MM.YYYY'),
           USER,
           SYSDATE,
           '&description (Script version &version)'
          );

   DBMS_OUTPUT.PUT_LINE ('-------------------------');
   DBMS_OUTPUT.PUT_LINE ('End   ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.PUT_LINE ('-------------------------');

   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('Please check in the log file, whether all the');
   DBMS_OUTPUT.PUT_LINE ('database objects were created successfully.  ');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If yes, please C O M M I T all the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If not, please R O L L B A C K the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');

EXCEPTION

   WHEN OTHERS THEN

      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      ROLLBACK;
      DBMS_OUTPUT.PUT_LINE ('   ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');
      DBMS_OUTPUT.PUT_LINE ('Because of error ... R O L L E D   B A C K ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');

END;
/

SPOOL Off
Set SERVEROUTPUT Off
Set VERIFY On
Set TRIMSPOOL Off
UNDEF scrip_parameter

UNDEF TPP_OWNER
UNDEF TPP_USER

UNDEF MEDIUM_DATA
UNDEF TEMP_TABLESPACE_DATA
UNDEF MEDIUM_IDX

UNDEF SMALL_DATA
UNDEF SMALL_IDX

UNDEF TPP_USER_PASS_SUFFIX
UNDEF TPP_USER_PASS

UNDEF patch_type
UNDEF patch_number
UNDEF rest_of_name
UNDEF step_order
UNDEF description
UNDEF version
UNDEF created_by
UNDEF create_date
UNDEF affected_apps
