-- DESCRIBING SECTION =============================================================================
-------------------------------------------------------------------------------- 
-- 1. SOFTWAROVA s.r.o.
-- (c) 2012
--------------------------------------------------------------------------------
-- TRANSACTIONAL PARALLEL PROCESSING
--------------------------------------------------------------------------------
-- Requirements:
-- Oracle Database Enterprise version >= 11g
-- CPU >= 1, Mem, Cluster
-- Sufficient size of REDO LOG is required.
--------------------------------------------------------------------------------
-- Run as user defined in TPP_USER
-- =============================================================================
------------- S C R I P T    N A M E -----------
DEF patch_type   = du
DEF patch_number = 00000001
DEF step_order   = 01
DEF rest_of_name = test_install_and_run_tpp_tppusr
------------- S C R I P T    I N F O -----------
DEF description     = 'Installation test script TPP'
DEF version         = 1.0
DEF created_by      = '1SW'
DEF create_date     = 07.06.2012
DEF affected_apps   = 'TPP'
REM for             = SQL*PLUS
REM parameters      = none

----------- S C R I P T    H I S T O R Y --------
REM     Version   When             Who               What
REM     1.0       2012/05/25       Petr Jezek        Creation
-- INITIAL UNIVERSAL SECTION ====================================================

@../du00000001_setting_env_tpp.sql

DEF script_name  = &patch_type&patch_number._&step_order&rest_of_name
Set SERVEROUTPUT On Size 1000000
SPOOL &script_name..log
Set VERIFY Off
Set TRIMSPOOL On
PROMPT
PROMPT *************************************************************************
PROMPT SCRIPT FILE: &script_name..sql
PROMPT LOG    FILE: &script_name..log
PROMPT SCRIPT DESC: &description
PROMPT SCRIPT VER.: &version
PROMPT *************************************************************************
PROMPT
PROMPT Affected Application(s) : &affected_apps
PROMPT Executing ...
PROMPT
-- EXECUTIVE SECTION ==============================================================================
Set FEEDBACK Off
Declare
  v_db_user VARCHAR2(500);
  v_net_user VARCHAR2(500);
Begin 
   -- Common actions
   SELECT 'Database user: ' || user || '@' || sys_context('USERENV', 'DB_NAME')
     INTO v_db_user 
     FROM dual;
   SELECT 'Network user: ' || sys_context('USERENV', 'OS_USER') || '@' || sys_context('USERENV', 'HOST')
     INTO v_net_user 
     FROM dual;
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('Start ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.put_line(v_db_user);
   DBMS_OUTPUT.put_line(v_net_user);   
   DBMS_OUTPUT.put_line ('------------------------------------------------------------');
   DBMS_OUTPUT.put_line ('	') ;
End; 
/
Set FEEDBACK On
Set DEF on
SET SERVEROUTPUT ON SIZE UNLIMITED

prompt create table test_to_process
create table test_to_process (
  x       number,
  y       varchar2(100),
  node_id number
)
tablespace &&SMALL_DATA;

comment on table test_to_process is
'Test table to transction on TPP processing example'
;
comment on column test_to_process.x is 'An value known as X number type';
comment on column test_to_process.y is 'An value known as Y varchar type';
comment on column test_to_process.node_id is 'Identification of node';

create or replace procedure to_execute (x in number, y in varchar2, node_id in number default 0)
as
 p    varchar2(100) := 'to_execute';
begin

    insert into test_to_process (x, y, node_id) values (to_number(x), y, node_id);
    dbms_lock.sleep(20);

exception
    when others then
       &&TPP_OWNER..tpp.add_log(p, 'err: '||to_char(sqlcode)||' '||dbms_utility.format_error_backtrace);
end to_execute;
/

grant execute on to_execute to &&TPP_OWNER;



declare
    l_task varchar2(30) := 'test_task';
    l_sql_chunk_stmt varchar2(32767);

    l_sql_execute_stmt varchar2(32767);
    l_try number;
    l_dbms_status number;

BEGIN

    -- Procedure to be execute in parallel
    l_sql_execute_stmt := 'begin &&TPP_USER..to_execute (1, ''test'', :node_id); end;';


    &&TPP_OWNER..tpp.initiate(
       p_task_name      => l_task
      ,p_comment        => 'The first try the Transactional Parallel Processing'
      ,p_sql_stmt       => l_sql_execute_stmt
      ,p_parallel_level => 4
    );


    &&TPP_OWNER..tpp.wait_for_end_of_processing(l_task);


    rollback;
    --commit;

    --
    &&TPP_OWNER..tpp.finalization(l_task);

END;
/

prompt Result after rollback 
set pagesize 100
column x format 9999
column y format a20
column node_id format 9999

select * from test_to_process;

declare
    l_task varchar2(30) := 'test_task';
    l_sql_chunk_stmt varchar2(32767);

    l_sql_execute_stmt varchar2(32767);
    l_try number;
    l_dbms_status number;

BEGIN

    -- Procedure to be execute in parallel
    l_sql_execute_stmt := 'begin &&TPP_USER..to_execute (1, ''test'', :node_id); end;';


    &&TPP_OWNER..tpp.initiate(
       p_task_name      => l_task
      ,p_comment        => 'Prvni pokus o parallel processing'
      ,p_sql_stmt       => l_sql_execute_stmt
      ,p_parallel_level => 4
    );


    &&TPP_OWNER..tpp.wait_for_end_of_processing(l_task);


    --rollback;
    commit;

    --
    &&TPP_OWNER..tpp.finalization(l_task);

END;
/

prompt Result after commit
set pagesize 100
column x format 9999
column y format a20
column node_id format 9999

select * from test_to_process;


Set DEF On

PROMPT *************************************************************************
BEGIN

-- FINAL UNIVERSAL SECTION ========================================================================


   DBMS_OUTPUT.PUT_LINE ('-------------------------');
   DBMS_OUTPUT.PUT_LINE ('End   ' || TO_CHAR(SYSDATE,'DD.MM.YYYY HH24:MI:SS') ) ;
   DBMS_OUTPUT.PUT_LINE ('-------------------------');

   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('Please check in the log file, whether all the');
   DBMS_OUTPUT.PUT_LINE ('database objects were created successfully.  ');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If yes, please C O M M I T all the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');
   DBMS_OUTPUT.PUT_LINE ('* If not, please R O L L B A C K the changes *');
   DBMS_OUTPUT.PUT_LINE ('**********************************************');

EXCEPTION

   WHEN OTHERS THEN

      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      ROLLBACK;
      DBMS_OUTPUT.PUT_LINE ('   ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');
      DBMS_OUTPUT.PUT_LINE ('Because of error ... R O L L E D   B A C K ');
      DBMS_OUTPUT.PUT_LINE ('*******************************************');

END;
/

SPOOL Off
Set SERVEROUTPUT Off
Set VERIFY On
Set TRIMSPOOL Off
UNDEF scrip_parameter



UNDEF patch_type
UNDEF patch_number
UNDEF rest_of_name
UNDEF step_order
UNDEF description
UNDEF version
UNDEF created_by
UNDEF create_date
UNDEF affected_apps
